/*
 * sxid - suid, sgid file and directory checking
 * sxid.c - main program routines
 *
 * Copyright (C) 1998 Ben Collins <bcollins@debian.org>
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with sxid; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/stat.h>
#include <time.h>

#include "config.h"
#include "structs.h"
#include "globals.h"

#ifndef HAVE_GETOPT
#include "getopt.h"
#else
#include <getopt.h>
#endif /* HAVE_GETOPT */

#include "sxid.h"
#include "conffile.h"
#include "search.h"
#include "notify.h"
#include "logging.h"

char SXID_VERSION[] = "v3.1.7";

int changed = 0;
int notify_diff = 0;
int nomail = 0;

int main (int argc, char *argv[])
{
    FILE *conffile;
    int opt, optindex = 0;
    time_t time_mark;

    umask (077);

    while ((opt = getopt_long_only (argc, argv, "c", cmd_options, &optindex)) != EOF) {
	switch (opt) {
	case 'c':
	    if (optarg != NULL)
		confname = optarg;
	    else
		sxid_usage ();
	    break;
	case 'n':
	    nomail = 1;
	    break;
	default:
	    sxid_usage ();
	}
    }
    argc -= optind;
    argv += optind;

    if ((conffile = fopen (confname, "r")) == NULL) {
	fprintf (stderr, "Failed to open configuration file %s\n", confname);
	exit (1);
    }
    fclose (conffile);

    /* Ok, let's time stamp */
    time_mark = time (NULL);
    if (!strftime (current_time, sizeof (current_time), "%c", localtime (&time_mark)))
	current_time[sizeof (current_time) - 1] = '\0';

    init_conf ();
    init_file_entries ();
    start_search ();
    compare_output ();

    if (changed || (config_options.flags & FLAG_AROT))
	save_and_rotate ();
    if (changed || notify_diff || nomail ||
	(config_options.flags & (FLAG_ANOT | FLAG_LALL)))
	notify_email ();

    fclose (mail_file);
    return 0;
}

void sxid_usage (void)
{
    fprintf (stdout, "sXid %s by Ben Collins - (c) 1998\n", SXID_VERSION);
    fprintf (stdout, "Usage: sxid [ --config <file> ] [ --nomail ]\n");
    exit (1);
}
